﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Configuration;
using MintXCore;

namespace MailTrail2
{
    partial class MintChipForm
    {
        delegate void PrecomputeChipDetailsDelegate();
        delegate void GetMintXMailsDelegate(bool forceReconnect);
        delegate void UpdateMailProgressBarDelegate();
        delegate void GetTransactionsDelegate(int number, bool forceReconect);
        delegate void GetTransactionsInDateRangeDelegate(DateTime fromm, DateTime to);
        delegate void UpdateTransactionProgressBarDelegate();

        List<MintXCore.ChipHandler> possibleChipHandlers;
        private static bool finishedLookingForChips;

        void PrecomputeChipDetails()
        {
            int chipTypes = this.ChipTypeComboBox.Items.Count;
            possibleChipHandlers = new List<ChipHandler>();
            for (int i = 0; i < chipTypes; i++)
            {
                string candidateChipType = this.ChipTypeComboBox.Items[i].ToString();
                MintXCore.ChipHandler tempChipHandler = MintXCore.MintChip.PreemptivelyGetChipHandlerInstance(config, candidateChipType);
                if (tempChipHandler != null)
                    possibleChipHandlers.Add(tempChipHandler);
            }
        }

        static void ConfirmFinishedLookingForChips(IAsyncResult result)
        {
            finishedLookingForChips = true;
        }

        void AsyncLookingForChips()
        {
            PrecomputeChipDetailsDelegate locateChips = new PrecomputeChipDetailsDelegate(PrecomputeChipDetails);
            locateChips.BeginInvoke(new AsyncCallback(ConfirmFinishedLookingForChips), null);
        }

        void AsyncGetMintXMails(bool forceReconnect)
        {
            GetMintXMailsDelegate getMails = new GetMintXMailsDelegate(GetMintXMails);
            getMails.BeginInvoke(forceReconnect, new AsyncCallback(ConfirmFinishedLookingForEmails), getMails);
            if (this.tabControl.SelectedTab == this.Yay)
            {
                this.YayProgressBar.Visible = true;
                this.YayRefreshButton.Visible = false;
                this.RequestButton.Visible = false;
            }
            else if (this.tabControl.SelectedTab == this.Okay)
            {
                this.OkayProgressBar.Visible = true;
                this.OkayRefreshButton.Visible = false;
                this.ContributeButton.Visible = false;
            }
        }

        void UpdateMailProgressBar()
        {
            if (this.tabControl.SelectedTab == this.Yay)
            {
                this.YayRefreshButton.Visible = true;
                this.RequestButton.Visible = true;
                this.YayProgressBar.Visible = false;
                MintChipForm_Yay_Enter();
            }
            else if (this.tabControl.SelectedTab == this.Okay)
            {
                this.OkayProgressBar.Visible = false;
                this.OkayRefreshButton.Visible = true;
                this.ContributeButton.Visible = true;
                MintChipForm_Okay_Enter();
            }
        }

        void ConfirmFinishedLookingForEmails(IAsyncResult result)
        {
            UpdateMailProgressBarDelegate update = new UpdateMailProgressBarDelegate(UpdateMailProgressBar);
            this.YayProgressBar.BeginInvoke(update);
        }

        void GetMintXMails(bool forceReconnect)
        {
            if ((paymentsList == null) || (forceReconnect))
            {
                parsedEmailList = new List<string>();
                paymentsList = mail.ConnectToGetPaymentsPosted(parsedEmailList, forceReconnect);
                requestsList = mail.ConnectToGetPaymentsRequested(false);
                paymentsList = FilterPaymentsForSelectedChip();
                requestsList = FilterRequestsForSelectedChip();
            }
        }

        void AsyncGetMintXChipTransactions(int number, bool forceReconnect)
        {
            this.TallyProgressBar.Visible = true;
            this.TallyHistoryLabel.Text = "Recent transactions";
            GetTransactionsDelegate getTransactions = new GetTransactionsDelegate(GetMintXChipTransactions);
            getTransactions.BeginInvoke(number, forceReconnect, new AsyncCallback(ConfirmFinishedLookingForTransactions), getTransactions);
        }

        void AsyncGetMintXChipTransactions(DateTime from, DateTime to)
        {
            this.TallyProgressBar.Visible = true;
            this.TallyHistoryLabel.Text = "Transactions from " + from.ToShortDateString() + " to " + to.ToShortDateString(); ;
            GetTransactionsInDateRangeDelegate getTransactions = new GetTransactionsInDateRangeDelegate(GetMintXChipTransactions);
            getTransactions.BeginInvoke(from, to, new AsyncCallback(ConfirmFinishedLookingForTransactions), getTransactions);
        }

        void ConfirmFinishedLookingForTransactions(IAsyncResult result)
        {
            UpdateTransactionProgressBarDelegate update = new UpdateTransactionProgressBarDelegate(UpdateTransactionProgressBar);
            this.TallyProgressBar.BeginInvoke(update);
        }

        void GetMintXChipTransactions(int number, bool forceReconnect = false)
        {
            if ((transactionHistory == null) || (forceReconnect))
            {
                transactionHistory = chipHandler.GetTransactions(number);
            }
        }

        void GetMintXChipTransactions(DateTime from, DateTime to)
        {
            transactionHistory = chipHandler.GetTransactions(from, to);
        }

        void UpdateTransactionProgressBar()
        {
            this.TallyProgressBar.Visible = false;
            this.TallyChipIDlabel.Visible = true;
            this.TallyChipIdTextBox.Visible = true;
            this.TallyBalanceLabel.Visible = true;
            this.TallyBalanceTextBox.Visible = true;
            this.TallyHistoryLabel.Visible = true;
            //this.TallyTransactionsRichTextBox.Visible = true;
            this.TallyTransactionsDataGridView.Visible = true;
            this.TallyFromDateTimePicker.Visible = true;
            this.TallyToDateTimePicker.Visible = true;
            this.TallyListLabel.Visible = true;
            this.TallyToLabel.Visible = true;
            this.TallyDateRangeButton.Visible = true;
            this.TallyRefreshButton.Visible = true;
            MintChipForm_Tally_Enter();
        }
    }
}
