﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Drawing;
using MintXCore;

namespace MailTrail2
{
    partial class MintChipForm
    {
        private int YaySelectedIndex = -1;
        private string YayTypedText = string.Empty;

        private void InitializePayments()
        {
            this.YayRefreshButton.Click += new EventHandler(YayRefreshButton_Click);
            this.RequestButton.Click += new EventHandler(RequestButton_Click);
            this.YayAcceptMoneyButton.Click += new EventHandler(YayAcceptMoneyButton_Click);
            this.YayAcceptAndThanksButton.Click += new EventHandler(YayAcceptAndThanksButton_Click);
            this.YaySendRequestButton.Click += new EventHandler(YaySendRequestButton_Click);
            this.YayToComboBox.TextChanged += new EventHandler(YayToComboBox_TextChanged);
            this.YaySenderListBox.DrawMode = DrawMode.OwnerDrawFixed;
            this.YaySenderListBox.DrawItem += new DrawItemEventHandler(YaySenderListBox_DrawItem);
        }

        void YaySenderListBox_DrawItem(object sender, DrawItemEventArgs e)
        {
            e.DrawBackground();
            MintChipMailData data = (MintChipMailData)this.YaySenderListBox.Items[e.Index];
            if (data.processedAlready)
            {
                e.Graphics.DrawString(data.listEntryRepresentation, e.Font, Brushes.Black, e.Bounds);
            }
            else
            {
                Font font = new Font(e.Font, FontStyle.Bold);
                e.Graphics.DrawString(data.listEntryRepresentation, font, Brushes.Black, e.Bounds);
            }
            e.DrawFocusRectangle();
        }

        void YayToComboBox_TextChanged(object sender, EventArgs e)
        {
            if (string.Equals(YayTypedText, this.YayToComboBox.Text)) return;

            if ((YaySelectedIndex != this.YayToComboBox.SelectedIndex) && (this.YayToComboBox.SelectedIndex >= 0))
            {
                this.YayToComboBox.TextChanged -= new EventHandler(YayToComboBox_TextChanged);
                this.YayToComboBox.DroppedDown = true;
                this.YayToComboBox.Text = this.YayTypedText;
                this.YayToComboBox.SelectionStart = this.YayTypedText.Length;
                this.YayToComboBox.TextChanged += new EventHandler(YayToComboBox_TextChanged);
                YaySelectedIndex = this.YayToComboBox.SelectedIndex;
                return;
            }

            this.YayTypedText = this.YayToComboBox.Text;
            string[] candidates = null;
            if (!string.IsNullOrEmpty(this.YayTypedText))
                candidates = this.parsedEmailList.Where(x => x.StartsWith(this.YayToComboBox.Text)).ToArray();
            if ((candidates == null) || (candidates.Length == 0))
            {
                this.YayToComboBox.DroppedDown = false;
            }
            else
            {
                this.YayToComboBox.TextChanged -= new EventHandler(YayToComboBox_TextChanged);
                this.YayToComboBox.DataSource = candidates;
                this.YayToComboBox.Text = this.YayTypedText;
                this.YayToComboBox.DroppedDown = true;
                this.YayToComboBox.SelectionStart = this.YayTypedText.Length;
                this.YayToComboBox.TextChanged += new EventHandler(YayToComboBox_TextChanged);
            }
        }

        void RequestButton_Click(object sender, EventArgs e)
        {
            foreach (TabPage tp in this.YayTabControl.TabPages)
            {
                this.YayTabControl.TabPages.Remove(tp);
            }
            this.YayTabControl.TabPages.Add(this.Please);
            this.Please.Text = "Pretty please";
            System.Resources.ResourceManager rm = new System.Resources.ResourceManager(typeof(Resource));
            this.YayRequestMessageTextBox.Text = rm.GetString("RequestMailText");
            this.YayToComboBox.Text = string.Empty;
            this.YayAmountRequestedTextBox.Text = string.Empty;
        }

        void YayRefreshButton_Click(object sender, EventArgs e)
        {
            MintChipForm_Yay_Selecting(forceReconnect: true);
        }

        void MintChipForm_Yay_Enter()
        {
            if ((mail == null) || (chipHandler == null)) return;
            if (paymentsList.Count == 0)
            {
                this.YayTabControl.TabPages.Add(this.Uhoh);
            }
            else
            {
                this.YaySenderListBox.DataSource = paymentsList;
                this.YaySenderListBox.DisplayMember = "listEntryRepresentation";
                this.YaySenderListBox.SelectedIndexChanged += new EventHandler(YaySenderListBox_SelectedIndexChanged);
                this.YayTabControl.TabPages.Add(this.Sweet);
                YaySenderListBox_SelectedIndexChanged(null, null);
            }
        }

        void YayAcceptAndThanksButton_Click(object sender, EventArgs e)
        {
            YayAcceptMoney();
            if (string.IsNullOrEmpty(this.SettingsErrorProvider.GetError(this.YayAmountReceivedTextBox)) &&
                string.IsNullOrEmpty(this.SettingsErrorProvider.GetError(this.YayAcceptMoneyButton)))
            {
                RequestButton_Click(null, null);
                this.Please.Text = "Thanks!";
                MintChipMailData data = (MintChipMailData)this.YaySenderListBox.SelectedItem;
                this.YayToComboBox.Text = data.fromEmail;
                System.Resources.ResourceManager rm = new System.Resources.ResourceManager(typeof(Resource));
                if (data.paymentType == MintChipMailData.PaymentType.CounterOfferred)
                {
                    this.YayRequestMessageTextBox.Text = rm.GetString("SettleMailText");
                    this.YayAmountRequestedTextBox.Text = this.YayAmountReceivedTextBox.Text.Substring("Offer of ".Length);
                }
                else
                {
                    this.YayRequestMessageTextBox.Text = rm.GetString("ThanksMailText");
                    this.YayAmountRequestedTextBox.Text = "0.00";
                }
            }
        }

        void YayAcceptMoneyButton_Click(object sender, EventArgs e)
        {
            YayAcceptMoney();
            if (string.IsNullOrEmpty(this.SettingsErrorProvider.GetError(this.YayAmountReceivedTextBox)) &&
                string.IsNullOrEmpty(this.SettingsErrorProvider.GetError(this.YayAcceptMoneyButton)))
            {
                MintChipForm_Yay_Selecting(forceReconnect: false);
            }
        }

        private void YayAcceptMoney()
        {
            string text = this.YayAmountReceivedTextBox.Text;
            int centValue = ParseTextToExtractMoney(text);
            MintChipMailData data = (MintChipMailData)this.YaySenderListBox.SelectedItem;
            if (centValue < 0)
            {
                if (data.paymentType != MintChipMailData.PaymentType.CounterOfferred)
                {
                    this.SettingsErrorProvider.SetError(this.YayAmountReceivedTextBox, "Bad monetary value prevented payment");
                    return;
                }
            }
            else if (!string.IsNullOrEmpty(this.SettingsErrorProvider.GetError(this.YayAmountReceivedTextBox)))
            {
                this.SettingsErrorProvider.Clear();
            }
            string returnVal = chipHandler.AcceptPayment(data.attachmentFilePath);
            if (!string.IsNullOrEmpty(returnVal))
            {
                this.SettingsErrorProvider.SetError(this.YayAcceptMoneyButton, returnVal);
                return;
            }
            data.processedAlready = true;
        }

        void YaySendRequestButton_Click(object sender, EventArgs e)
        {
            string text = this.YayAmountRequestedTextBox.Text;
            int centValue = ParseTextToExtractMoney(text);
            if (centValue < 0)
            {
                this.SettingsErrorProvider.SetError(this.YayAmountRequestedTextBox, "Put in legitimate monetary figures like '15' or '9.24' etc.");
                return;
            }
            else
            {
                if (!string.IsNullOrEmpty(this.SettingsErrorProvider.GetError(this.YayAmountRequestedTextBox)))
                {
                    this.SettingsErrorProvider.Clear();
                }
            }

            this.YaySendRequestButton.Enabled = false;
            string annotationMsg = "Request for $" + centValue.ToMoney() + " from " + this.UsernameTextbox.Text;
            if (annotationMsg.Length > 64)
                annotationMsg = annotationMsg.Substring(0, 64);
            string responseAddress = @"<a href='mailto:" + this.UsernameTextbox.Text + @"'>";
            string response = this.chipHandler.RequestPayment(centValue, annotationMsg, responseAddress);
            if (System.IO.File.Exists(response))
            {
                MintChipMailData data = new MintChipMailData
                {
                    fromEmail = this.UsernameTextbox.Text,
                    toEmail = this.YayToComboBox.Text,
                    body = this.YayRequestMessageTextBox.Text,//msg.FindFirstPlainTextVersion().GetBodyAsText(),
                    subject = "Hi from " + this.UsernameTextbox.Text + " (via MintX)",
                    attachmentFilePath = response
                };
                response = mail.ConnectToSendRequestForPayment(data);
                if (!string.IsNullOrEmpty(response))
                {
                    this.SettingsErrorProvider.SetError(this.YaySendRequestButton, "Unable to send mail: " + response);
                }
            }
            else
            {
                this.SettingsErrorProvider.SetError(this.YaySendRequestButton, "Unable to create a mintchip request: " + response);
            }
            this.YaySendRequestButton.Enabled = true;
            if (string.IsNullOrEmpty(this.SettingsErrorProvider.GetError(this.YaySendRequestButton)))
            {

                MintChipForm_Yay_Selecting(forceReconnect: false);
            }
        }

        void YaySenderListBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            MintChipMailData data = (MintChipMailData)this.YaySenderListBox.SelectedItem;
            this.YaySenderMessageTextBox.Text = data.body;
            this.YayAmountReceivedTextBox.Text = chipHandler.GetOfferredAmount(data.attachmentFilePath, true);
            if (data.processedAlready)
            {
                this.YayAcceptMoneyButton.Enabled = false;
                this.YayAcceptMoneyButton.Text = "Accepted";
                this.YayAcceptAndThanksButton.Text = "Thank";
                if (data.paymentType == MintChipMailData.PaymentType.CounterOfferred)
                {
                    this.YayAcceptAndThanksButton.Text = "Follow up";
                }
            }
            else
            {
                this.YayAcceptMoneyButton.Enabled = true;
                this.YayAcceptMoneyButton.Text = "Accept";
                this.YayAcceptAndThanksButton.Text = "Accept and thank";
                if (data.paymentType == MintChipMailData.PaymentType.CounterOfferred)
                {
                    this.YayAcceptMoneyButton.Enabled = false;
                    this.YayAcceptAndThanksButton.Text = "Settle, Renegotiate";
                }
            }
        }

        void MintChipForm_Yay_Selecting(bool forceReconnect)
        {
            foreach (TabPage tp in this.YayTabControl.TabPages)
            {
                this.YayTabControl.TabPages.Remove(tp);
            }
            this.YayTabControl.Visible = true;
            System.Resources.ResourceManager rm = new System.Resources.ResourceManager(typeof(Resource));
            if ((mail == null) || (chipHandler == null))
            {
                this.YayUhOhRichTextBox.Rtf = rm.GetString("PleaseLoginText");
                this.YayTabControl.TabPages.Add(this.Uhoh);
            }
            else
            {
                this.YayUhOhRichTextBox.Rtf = rm.GetString("YayText");
                this.YayRefreshButton.Visible = true;
                this.RequestButton.Visible = true;
                AsyncGetMintXMails(forceReconnect);
            }
        }
    }
}
