﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Text.RegularExpressions;
using System.Drawing;
using MintXCore;

namespace MailTrail2
{
    partial class MintChipForm
    {
        enum MailScenario
        {
            Contribute,
            Pay,
            Haggle,
            FollowUp
        }

        private MailScenario mailScenario;
        private MintChipMailData mailData;

        private void InitializeRequests()
        {
            this.OkayRefreshButton.Click += new EventHandler(OkayRefreshbutton_Click);
            this.ContributeButton.Click += new EventHandler(ContributeButton_Click);
            this.OkayPayMoneybutton.Click += new EventHandler(OkayPayMoneybutton_Click);
            this.OkayHaggleButton.Click += new EventHandler(OkayHaggleButton_Click);
            this.OkaySendContributionButton.Click += new EventHandler(OkaySendContributionButton_Click);
            this.OkaySenderListBox.DrawMode = DrawMode.OwnerDrawFixed;
            this.OkaySenderListBox.DrawItem += new DrawItemEventHandler(OkaySenderListBox_DrawItem);
            this.OkayIgnoreButton.Click += new EventHandler(OkayIgnoreButton_Click);
        }

        void OkayIgnoreButton_Click(object sender, EventArgs e)
        {
            if (mailScenario != MailScenario.Contribute)
            {
                mailData = (MintChipMailData)this.OkaySenderListBox.SelectedItem;
                mailData.paymentType = MintChipMailData.PaymentType.Ignored;
                mailData.processedAlready = true;
                string ignoredMails = string.Empty;
                if (config.AppSettings.Settings["ignoredEmails"] != null)
                {
                    ignoredMails = config.AppSettings.Settings["ignoredEmails"].Value + ",";
                    config.AppSettings.Settings.Remove("ignoredEmails");
                }
                ignoredMails += mailData.attachmentFilePath;
                config.AppSettings.Settings.Add("ignoredEmails", ignoredMails);
            }
            MintChipForm_Okay_Selecting(forceReconnect: false);
        }

        void OkayAmountContributedTextBox_Click(object sender, EventArgs e)
        {
            System.Resources.ResourceManager rm = new System.Resources.ResourceManager(typeof(Resource));
            string toolTipMessage = rm.GetString("HaggleTooltipText");
            MessageBox.Show(toolTipMessage);
        }

        void OkaySenderListBox_DrawItem(object sender, DrawItemEventArgs e)
        {
            e.DrawBackground();
            MintChipMailData data = (MintChipMailData)this.OkaySenderListBox.Items[e.Index];
            if (data.processedAlready)
            {
                e.Graphics.DrawString(data.listEntryRepresentation, e.Font, Brushes.Black, e.Bounds);
            }
            else
            {
                Font font = new Font(e.Font, FontStyle.Bold);
                e.Graphics.DrawString(data.listEntryRepresentation, font, Brushes.Black, e.Bounds);
            }
            e.DrawFocusRectangle();
        }

        void OkaySendContributionButton_Click(object sender, EventArgs e)
        {
            if ((mailScenario == MailScenario.Pay) && (!string.Equals(this.OkayAmountContributedTextBox.Text, this.OkayAmountRequestedTextBox.Text)))
            {
                mailScenario = MailScenario.Haggle;
            }
            this.OkaySendContributionButton.Enabled = false;
            string text = this.OkayAmountContributedTextBox.Text;
            int centValue = ParseTextToExtractMoney(text);
            if (centValue < 0)
            {
                this.SettingsErrorProvider.SetError(this.OkayAmountContributedTextBox, "Put in legitimate monetary figures like '15' or '9.24' etc.");
                this.OkaySendContributionButton.Enabled = true;
                return;
            }
            else if (!string.IsNullOrEmpty(this.SettingsErrorProvider.GetError(this.OkayAmountContributedTextBox)))
            {
                this.SettingsErrorProvider.Clear();
            }
            OkayPayMoney(centValue, false);
            this.OkaySendContributionButton.Enabled = true;
            if (string.IsNullOrEmpty(this.SettingsErrorProvider.GetError(this.OkaySendContributionButton)))
            {
                MintChipForm_Okay_Selecting(forceReconnect: false);
            }
        }

        void OkayHaggleButton_Click(object sender, EventArgs e)
        {
            ContributeButton_Click(null, null);
            this.OkayHowAboutTabPage.Text = "How about..";
            System.Resources.ResourceManager rm = new System.Resources.ResourceManager(typeof(Resource));
            mailData = (MintChipMailData)this.OkaySenderListBox.SelectedItem;
            this.OkayToTextBox.Text = mailData.fromEmail;
            if (!mailData.processedAlready)
            {
                this.OkayAmountContributedTextBox.Text = this.OkayAmountRequestedTextBox.Text;
                this.OkayAmountContributedTextBox.Click += new EventHandler(OkayAmountContributedTextBox_Click);
                this.OkayHaggleMessageTextBox.Text = rm.GetString("HaggleMailText");
                mailScenario = MailScenario.Pay;
            }
            else
            {
                this.OkayAmountContributedTextBox.Text = "0.00";
                this.OkayHaggleMessageTextBox.Text = rm.GetString("FollowUpMailText");
                mailScenario = MailScenario.FollowUp;
            }
        }

        void OkayPayMoneybutton_Click(object sender, EventArgs e)
        {
            this.OkayPayMoneybutton.Enabled = false;
            mailScenario = MailScenario.Pay;
            string text = this.OkayAmountRequestedTextBox.Text;
            int centValue = ParseTextToExtractMoney(text);
            if (centValue < 0)
            {
                this.SettingsErrorProvider.SetError(this.OkayAmountRequestedTextBox, "Put in legitimate monetary figures like '15' or '9.24' etc.");
                this.OkayPayMoneybutton.Enabled = true;
                return;
            }
            else if (!string.IsNullOrEmpty(this.SettingsErrorProvider.GetError(this.OkayAmountRequestedTextBox)))
            {
                this.SettingsErrorProvider.Clear();
            }
            OkayPayMoney(centValue, true);
            this.OkayPayMoneybutton.Enabled = true;
            if (string.IsNullOrEmpty(this.SettingsErrorProvider.GetError(this.OkayPayMoneybutton)))
            {
                MintChipForm_Okay_Selecting(forceReconnect: false);
            }
        }

        private void OkayPayMoney(int centValue, bool fromDirectPaymentButton)
        {
            string msgSubject = string.Empty;
            string annotationMsg = "Contribution of $" + centValue.ToMoney() + " from " + this.UsernameTextbox.Text;
            mailData = (MintChipMailData)this.OkaySenderListBox.SelectedItem;
            string payeeId = mailData == null? string.Empty : mailData.payeeId;
            int challenge = mailData == null ? 0 : mailData.challenge;
            string mailTo = mailData == null ? string.Empty : mailData.fromEmail;
            string emailBody = fromDirectPaymentButton ? "MintX automated message: Payment sent" : this.OkayHaggleMessageTextBox.Text;
            switch (mailScenario)
            {
                case MailScenario.Pay:
                    msgSubject = "Money paid from " + this.UsernameTextbox.Text + " (via MintX)";
                    break;
                case MailScenario.Haggle:
                    msgSubject = "Counter offer from " + this.UsernameTextbox.Text + " (via MintX)";
                    annotationMsg = "COUNTER_OFFER_CENTS:" + centValue; // + ", ORIGINAL_REQUEST_CENTS:" + ParseTextToExtractMoney(this.OkayAmountRequestedTextBox.Text);
                    centValue = 1;
                    break;
                case MailScenario.FollowUp:
                    msgSubject = "Follow up from " + this.UsernameTextbox.Text + " (via MintX)";
                    break;
                case MailScenario.Contribute:
                    msgSubject = "Thanks from " + this.UsernameTextbox.Text + " (via MintX)";
                    payeeId = this.developerPayeeId;
                    mailTo = this.developerEmailId;
                    challenge = 0;
                    break;
            }

            if (annotationMsg.Length > 64)
                annotationMsg = annotationMsg.Substring(0, 64);
            string responseAddress = @"<a href='mailto:" + this.UsernameTextbox.Text + @"'>";
            string response = this.chipHandler.PayMoney(centValue, payeeId, annotationMsg, challenge);
            if (System.IO.File.Exists(response))
            {
                MintChipMailData data = new MintChipMailData
                {
                    fromEmail = this.UsernameTextbox.Text,
                    toEmail = mailTo,
                    body = emailBody,
                    subject = msgSubject, 
                    attachmentFilePath = response
                };
                response = mail.ConnectToSendPayment(data);
                if (!string.IsNullOrEmpty(response))
                {
                    if (fromDirectPaymentButton)
                        this.SettingsErrorProvider.SetError(this.OkayPayMoneybutton, "Unable to send mail: " + response);
                    else
                        this.SettingsErrorProvider.SetError(this.OkaySendContributionButton, "Unable to send mail: " + response);
                }
            }
            else
            {
                if (fromDirectPaymentButton)
                    this.SettingsErrorProvider.SetError(this.OkayPayMoneybutton, "Unable to create a mintchip payment: " + response);
                else
                    this.SettingsErrorProvider.SetError(this.OkaySendContributionButton, "Unable to create a mintchip payment: " + response);
            }
            if (mailScenario != MailScenario.Contribute)
            {
                mailData.processedAlready = true;
            }
        }

        void ContributeButton_Click(object sender, EventArgs e)
        {
            foreach (TabPage tp in this.OkayTabControl.TabPages)
            {
                this.OkayTabControl.TabPages.Remove(tp);
            }
            this.OkayTabControl.TabPages.Add(this.OkayHowAboutTabPage);
            this.OkayHowAboutTabPage.Text = "Gee, thanks!";
            this.OkayToTextBox.Text = this.developerEmailId;
            this.OkayHaggleAmountLabel.Text = "Amount contributed";
            this.OkayAmountContributedTextBox.Text = "1.00";
            System.Resources.ResourceManager rm = new System.Resources.ResourceManager(typeof(Resource));
            this.OkayHaggleMessageTextBox.Text = rm.GetString("ContributionMailText");
            this.OkayAmountContributedTextBox.Click -= new EventHandler(OkayAmountContributedTextBox_Click);
            mailScenario = MailScenario.Contribute;
        }

        void OkayRefreshbutton_Click(object sender, EventArgs e)
        {
            MintChipForm_Okay_Selecting(forceReconnect: true);
        }

        void MintChipForm_Okay_Enter()
        {
            if ((mail == null) || (chipHandler == null)) return;
            if (requestsList.Count == 0)
            {
                this.OkayTabControl.TabPages.Add(this.OkayGoodTabPage);
            }
            else
            {
                this.OkaySenderListBox.DataSource = requestsList;
                this.OkaySenderListBox.DisplayMember = "listEntryRepresentation";
                this.OkaySenderListBox.SelectedIndexChanged += new EventHandler(OkaySenderListBox_SelectedIndexChanged);
                this.OkayTabControl.TabPages.Add(this.OkayISeeTabPage);
                OkaySenderListBox_SelectedIndexChanged(null, null);
            }
        }

        void OkaySenderListBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            MintChipMailData data = (MintChipMailData)this.OkaySenderListBox.SelectedItem;
            this.OkaySenderMessageTextBox.Text = data.body;
            this.OkayAmountRequestedTextBox.Text = chipHandler.GetOfferredAmount(data.attachmentFilePath, false);
            if (data.processedAlready)
            {
                this.OkayPayMoneybutton.Enabled = false;
                this.OkayPayMoneybutton.Text = "Handled";
                this.OkayHaggleButton.Text = "Follow up";
            }
            else
            {
                this.OkayPayMoneybutton.Enabled = true;
                this.OkayPayMoneybutton.Text = "Pay";
                this.OkayHaggleButton.Text = "Negotiate, Confirm, Ignore";
            }
        }

        void MintChipForm_Okay_Selecting(bool forceReconnect)
        {
            foreach (TabPage tp in this.OkayTabControl.TabPages)
            {
                this.OkayTabControl.TabPages.Remove(tp);
            }
            this.OkayTabControl.Visible = true;
            System.Resources.ResourceManager rm = new System.Resources.ResourceManager(typeof(Resource));
            if ((mail == null) || (chipHandler == null))
            {
                this.OkayGoodRichTextBox.Rtf = rm.GetString("PleaseLoginToHelpText");
                this.OkayGoodTabPage.Text = "Umm..";
                this.OkayTabControl.TabPages.Add(this.OkayGoodTabPage);
            }
            else
            {
                this.OkayGoodRichTextBox.Rtf = rm.GetString("OkayText");
                this.OkayGoodTabPage.Text = "Good!";
                this.OkayHowAboutTabPage.Text = "How about...";
                this.OkayRefreshButton.Visible = true;
                this.ContributeButton.Visible = true;
                AsyncGetMintXMails(forceReconnect);
            }
        }
    }
}
