﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Security;
using System.Windows.Forms;
using MintXCore;

namespace MailTrail2
{
    partial class MintChipForm
    {
        private const string CONFIRMED = "-*-CONFIRMED-*-";
        private const string INCORRECTCHIPID = "XXXXXXXXXX";
        private SecureString password;
        private void InitializeSettings()
        {
            this.ShowPasswordButton.Click += new EventHandler(ShowPasswordButton_Click);
            this.EmailSelectorComboBox.SelectedIndexChanged += new EventHandler(EmailSelectorComboBox_SelectedIndexChanged);
            this.ChipTypeComboBox.SelectedIndexChanged += new EventHandler(ChipTypeComboBox_TabIndexChanged);
            this.LocateChipButton.Click += new EventHandler(LocateChipButton_Click);
            this.SaveConfigsButton.Click += new EventHandler(SaveConfigsButton_Click);
            this.SettingsHelpButton.Click += new EventHandler(SettingsHelpButton_Click);
        }

        private void ShowPasswordButton_Click(object sender, EventArgs e)
        {
            if (this.ShowPasswordButton.Text.ToLower() == "show")
            {
                this.ShowPasswordButton.Text = "Hide";
                this.PasswordTextbox.PasswordChar = (char)0;
            }
            else
            {
                this.ShowPasswordButton.Text = "Show";
                this.PasswordTextbox.PasswordChar = '*';
            }
            this.ShowPasswordButton.Enabled = true;
        }

        void EmailSelectorComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            this.SettingsErrorProvider.Clear();
            this.UsernameTextbox.Text = "";
            this.PasswordTextbox.Text = "";
            this.PasswordTextbox.ReadOnly = false;
            if (this.ShowPasswordButton.Text == "Hide") ShowPasswordButton_Click(null, null);
        }

        void SaveConfigsButton_Click(object sender, EventArgs e)
        {
            this.SettingsErrorProvider.Clear();
            ConfirmChipCreds();
            ConfirmEmailCreds();
            if ((mail != null) && (chipHandler != null))
            {
                this.tabControl.SelectTab("Yay");
            }
        }

        private void ConfirmEmailCreds()
        {
            SetEmailCredsInConfig();
            if ((!string.IsNullOrEmpty(this.SettingsErrorProvider.GetError(this.EmailSelectorComboBox))) ||
                (!string.IsNullOrEmpty(this.SettingsErrorProvider.GetError(this.UsernameTextbox))) ||
                (!string.IsNullOrEmpty(this.SettingsErrorProvider.GetError(this.PasswordTextbox))))
                return;
            mail = MintXCore.Mail.GetMailInstance(config, password);
            if (mail == null)
            {
                this.SettingsErrorProvider.SetError(this.UsernameTextbox, "Unable to connect to mail server. See help");
            }
            else
            {
                this.PasswordTextbox.Text = MintChipForm.CONFIRMED;
                ShowPasswordButton_Click(null, null);
                this.ShowPasswordButton.Enabled = false;
                this.PasswordTextbox.ReadOnly = true;
            }
        }

        private void SetEmailCredsInConfig()
        {
            string emailType = this.EmailSelectorComboBox.SelectedItem == null ? string.Empty : this.EmailSelectorComboBox.SelectedItem.ToString();
            string emailId = this.UsernameTextbox.Text;
            if (string.IsNullOrEmpty(emailType))
            {
                this.SettingsErrorProvider.SetError(this.EmailSelectorComboBox, "Select something maybe?");
                return;
            }
            else
            {
                if (string.Equals(emailType, "Windows Live ID", StringComparison.InvariantCultureIgnoreCase))
                    emailType = "Live";
                else
                    emailType = "Gmail";
            }

            if (string.IsNullOrEmpty(emailId.Trim()))
            {
                this.SettingsErrorProvider.SetError(this.UsernameTextbox, "We do need this, you know!");
                return;
            }
            if (string.IsNullOrEmpty(this.PasswordTextbox.Text.Trim()))
            {
                this.SettingsErrorProvider.SetError(this.PasswordTextbox, "You really don't want us to guess this one on our own.");
                return;
            }
            if (config.AppSettings.Settings["emailId"] != null)
            {
                config.AppSettings.Settings.Remove("emailId");
            }
            if (config.AppSettings.Settings["emailType"] != null)
            {
                config.AppSettings.Settings.Remove("emailType");
            }
            config.AppSettings.Settings.Add("emailId", emailId);
            config.AppSettings.Settings.Add("emailType", emailType);
            if (string.Equals(this.PasswordTextbox.Text, MintChipForm.CONFIRMED))
                return;
            char[] passwordChars = this.PasswordTextbox.Text.ToCharArray();
            password = new SecureString();
            foreach (char x in passwordChars)
            {
                password.AppendChar(x);
            }
        }

        void ChipTypeComboBox_TabIndexChanged(object sender, EventArgs e)
        {
            this.LocateChipButton.Visible = true;
            this.ChipIdTextBox.Text = "";
            this.SettingsErrorProvider.Clear();
        }

        void LocateChipButton_Click(object sender, EventArgs e)
        {
            ConfirmChipCreds();
        }

        private void ConfirmChipCreds()
        {
            SetChipDetailsInConfig();
            if (!string.IsNullOrEmpty(this.SettingsErrorProvider.GetError(ChipTypeComboBox)))
                return;
            while (!finishedLookingForChips) ;
            if (possibleChipHandlers.Count == 0)
            {
                this.SettingsErrorProvider.SetError(this.ChipIdTextBox, "Unable to locate remote or local chip. See help");
            }
            else
            {
                if (!string.IsNullOrEmpty(this.SettingsErrorProvider.GetError(this.ChipIdTextBox)))
                    this.SettingsErrorProvider.Clear();
                string selectedChipType = config.AppSettings.Settings["chipType"].Value;
                if (possibleChipHandlers.Count == 2) // found both remote and local chip
                {
                    if (selectedChipType.ToLowerInvariant() == "local")
                    {
                        chipHandler = possibleChipHandlers[0];
                    }
                    else
                    {
                        chipHandler = possibleChipHandlers[1];
                    }
                }
                else
                {
                    if (selectedChipType.ToLowerInvariant() == "local")
                    {
                        chipHandler = possibleChipHandlers[0] as LocalChipHandler;
                        if (chipHandler == null)
                        {
                            this.SettingsErrorProvider.SetError(this.ChipIdTextBox, "Unable to load local chip. Try the remote chip option or see help");
                        }
                    }
                    else
                    {
                        chipHandler = possibleChipHandlers[0] as RemoteChipHandler;
                        if (chipHandler == null)
                        {
                            this.SettingsErrorProvider.SetError(this.ChipIdTextBox, "Unable to load remote chip. Try the local chip option or see help");
                        }
                    }
                }
            }

            if (chipHandler == null)
            {
                this.ChipIdTextBox.Text = INCORRECTCHIPID;
            }
            else
            {
                this.ChipIdTextBox.Text = chipHandler.GetMintChipID();
            }

            this.ChipId.Visible = true;
            this.ChipIdTextBox.Visible = true;
        }


        void SettingsHelpButton_Click(object sender, EventArgs e)
        {
            Form f = new Form();
            f.Size = new System.Drawing.Size(800, 800);
            RichTextBox rtb = new RichTextBox();
            rtb.Dock = DockStyle.Fill;
            System.Resources.ResourceManager rm = new System.Resources.ResourceManager(typeof(Resource));
            rtb.Rtf = rm.GetString("SettingsHelpText");
            f.Controls.Add(rtb);
            f.Show();
        }

        private void SetChipDetailsInConfig()
        {
            string chipType = this.ChipTypeComboBox.SelectedItem == null ? string.Empty : this.ChipTypeComboBox.SelectedItem.ToString();
            if (string.IsNullOrEmpty(chipType))
            {
                this.SettingsErrorProvider.SetError(this.ChipTypeComboBox, "Select something maybe?");
                return;
            }
            if (config.AppSettings.Settings["chipType"] != null)
            {
                config.AppSettings.Settings.Remove("chipType");
            }
            config.AppSettings.Settings.Add("chipType", chipType);
        }

        void MintChipForm_Settings_Selecting()
        {
            AsyncLookingForChips();
        }
    }
}
