﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Configuration;
using System.Text.RegularExpressions;
using MintXCore;

/*
 * Icon information: 
 * http://www.codeproject.com/Articles/12634/How-to-Create-Icon-Files-for-Your-Windows-App-usin
 * http://msdn.microsoft.com/en-us/library/339stzf7.aspx (see community content)
 */

namespace MailTrail2
{
    public partial class MintChipForm : Form
    {
        private MintXCore.Mail mail;
        private MintXCore.ChipHandler chipHandler;
        private Configuration config;
        private List<MintChipMailData> paymentsList;
        private List<MintChipMailData> requestsList;
        private List<Transactions> transactionHistory;
        private List<string> parsedEmailList;
        private string developerPayeeId = "1310000000002603";
        private string developerEmailId = "mintxca@gmail.com";

        // http://msdn.microsoft.com/en-us/library/ms404305.aspx
        public MintChipForm()
        {
            InitializeComponent();
            config = ConfigurationManager.OpenExeConfiguration(ConfigurationUserLevel.None);
            this.tabControl.DrawItem += new DrawItemEventHandler(tabControl_DrawItem);
            IntializeMessages();
            InitializeSettings();
            InitializePayments();
            InitializeRequests();
            InitializeTally();
            this.tabControl.Selecting += new TabControlCancelEventHandler(tabControl_Selecting);
        }

        void tabControl_Selecting(object sender, TabControlCancelEventArgs e)
        {
            if (e.TabPage.Name == "Settings")
            {
                MintChipForm_Settings_Selecting();
            }
            else if (e.TabPage.Name == "Yay")
            {
                MintChipForm_Yay_Selecting(forceReconnect: false);
            }
            else if (e.TabPage.Name == "Okay")
            {
                MintChipForm_Okay_Selecting(forceReconnect: false);
            }
            else if (e.TabPage.Name == "Tally")
            {
                MintChipFrom_Tally_Selecting();
            }
        }

        void IntializeMessages()
        {
            System.Resources.ResourceManager rm = new System.Resources.ResourceManager(typeof(Resource));
            this.welcomeTextBox.Rtf = rm.GetString("WelcomeText");
            this.YayUhOhRichTextBox.Rtf = rm.GetString("YayText");
            this.OkayGoodRichTextBox.Rtf = rm.GetString("OkayText");
            this.WhatRichTextBox.Rtf = rm.GetString("WhatText");
            this.TallyRichTextBox.Rtf = rm.GetString("PleaseLoginToTotalText");
            //this.YayRequestMessageTextBox.Text = rm.GetString("RequestMailText");
        }

        void tabControl_DrawItem(object sender, DrawItemEventArgs e)
        {
            Graphics g = e.Graphics;
            Brush _textBrush;

            // Get the item from the collection.
            TabPage _tabPage = tabControl.TabPages[e.Index];

            // Get the real bounds for the tab rectangle.
            Rectangle _tabBounds = tabControl.GetTabRect(e.Index);

            if (e.State == DrawItemState.Selected)
            {

                // Draw a different background color, and don't paint a focus rectangle.
                _textBrush = new SolidBrush(Color.YellowGreen);
                g.FillRectangle(Brushes.Gray, e.Bounds);
            }
            else
            {
                _textBrush = new System.Drawing.SolidBrush(e.ForeColor);
                e.DrawBackground();
            }

            // Use our own font.
            Font _tabFont = new Font("Arial", (float)25.0, FontStyle.Bold, GraphicsUnit.Pixel);

            // Draw string. Center the text.
            StringFormat _stringFlags = new StringFormat();
            _stringFlags.Alignment = StringAlignment.Center;
            _stringFlags.LineAlignment = StringAlignment.Center;
            g.DrawString(_tabPage.Text, _tabFont, _textBrush, _tabBounds, new StringFormat(_stringFlags));
        }

        private void MintChipForm_Load(object sender, EventArgs e)
        {
            this.MaximizeBox = false;
        }

        private List<MintChipMailData> FilterPaymentsForSelectedChip()
        {
            List<MintChipMailData> filteredPayments = new List<MintChipMailData>();
            foreach (MintChipMailData data in paymentsList)
            {
                int amountInCents = chipHandler.GetAmountInCentsFromValueMessage(data.attachmentFilePath, true);
                if (amountInCents == 0) // money accepted already
                {
                    data.processedAlready = true;
                    data.paymentType = MintChipMailData.PaymentType.Paid;
                    filteredPayments.Add(data);
                }
                else if (amountInCents > 0) // new income
                {
                    data.processedAlready = false;
                    data.paymentType = MintChipMailData.PaymentType.Paid;
                    filteredPayments.Add(data);
                }
                else if (amountInCents == -2) // an offer to haggle
                {
                    data.processedAlready = false;
                    data.paymentType = MintChipMailData.PaymentType.CounterOfferred;
                    filteredPayments.Add(data);
                }
                else if (amountInCents == -3) // an offer to haggle
                {
                    data.processedAlready = true;
                    data.paymentType = MintChipMailData.PaymentType.CounterOfferred;
                    filteredPayments.Add(data);
                }
            }
            filteredPayments.Reverse(); // the way the mails are queried makes the original list reverse sorted. This fixes it
            return filteredPayments;
        }

        private List<MintChipMailData> FilterRequestsForSelectedChip()
        {
            List<MintChipMailData> filteredRequests = new List<MintChipMailData>();
            string[] ignoredMails = new string[] { };
            if (config.AppSettings.Settings["ignoredEmails"] != null)
            {
                ignoredMails = config.AppSettings.Settings["ignoredEmails"].Value.Split(new char[] { ',' });
            }
            foreach (MintChipMailData data in requestsList)
            {
                int amountInCents = chipHandler.GetAmountInCentsFromValueMessage(data.attachmentFilePath, false);
                if (amountInCents < 0)
                    continue;
                chipHandler.PopulatePaymentDataFromRequest(data);
                if (amountInCents == 0)
                {
                    data.processedAlready = true;
                }
                else if (amountInCents > 0)
                {
                    data.processedAlready = false;
                }
                if (ignoredMails.Contains(data.attachmentFilePath))
                {
                    data.processedAlready = true;
                    data.paymentType = MintChipMailData.PaymentType.Ignored;
                }
                filteredRequests.Add(data);
            }
            filteredRequests.Reverse(); // the way the mails are queried makes the original list reverse sorted. This fixes it
            return filteredRequests;
        }

        private int ParseTextToExtractMoney(string text)
        {
            text = text.Replace("$", "").Trim();
            Regex r1 = new Regex(@"^(\d+)\.(\d{2})$");
            Regex r2 = new Regex(@"^(\d+)$");
            Match m1 = r1.Match(text);
            Match m2 = r2.Match(text);
            if ((!m1.Success) && (!m2.Success))
            {
                return -1;
            }
            int centValue = 0;
            if (m1.Success)
            {
                centValue = 100 * int.Parse(m1.Groups[1].Value) + int.Parse(m1.Groups[2].Value);
            }
            if (m2.Success)
            {
                centValue = 100 * int.Parse(m2.Groups[0].Value);
            }
            return centValue;
        }
    }
}

