﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Configuration;
using System.Security;
using System.IO;
using System.Net.Mail;
using AE.Net.Mail.Imap;
using OpenPop.Pop3;

namespace MintXCore
{
    public class MintChipMailData
    {
        public enum PaymentType
        {
            Requested,
            Paid,
            CounterOfferred,
            Ignored
        };
        public string fromEmail { get; set; }
        public string fromName { get; set; }
        public DateTime onDate { get; set; }
        public string body { get; set; }
        public string attachmentFilePath { get; set; }
        public PaymentType paymentType { get; set; }
        public bool processedAlready { get; set; }
        public string messageId { get; set; }
        private string listRepresentation;
        public string listEntryRepresentation
        {
            get
            {
                if (processedAlready)
                {
                    if (paymentType == PaymentType.Ignored)
                        return listRepresentation + "    (ignored)";
                    return listRepresentation + "    (read)";
                }
                else
                {
                    if (paymentType == PaymentType.CounterOfferred)
                        return listRepresentation + "    (counter offer)";
                    return listRepresentation + "    NEW";
                }
            }
            set
            {
                listRepresentation = value;
            }
        }
        public string toEmail { get; set; }
        public string subject { get; set; }
        public string payeeId { get; set; }
        public int challenge { get; set; }
    }

    public class Mail
    {
        private int limit = 10;
        private string unknownConst = "UNKNOWN";
        private static string downloadDirectoryName = "DOWNLOADS";
        private string[] extensionsOfInterest;
        private string[] mimeTypesOfInterest;
        private string emailType;
        private string emailId;
        private string smtpHost;
        private int smtpPort;
        private string popOrImapHost;
        private int popOrImapPort;
        private Configuration config;
        private SecureString encryptedPassword;
        private static Mail mail;
        private OpenPop.Pop3.Pop3Client OpenPopClient;
        private List<MintChipMailData> RequestedPaymentMessages;
        private List<MintChipMailData> PaidPaymentMessages;
        private string downloadDirectoryPath;

        private enum PaymentType
        {
            Requested,
            Posted,
            All,
            None,
            ValidateOnly
        };

        public static Mail GetMailInstance(Configuration conf, SecureString password)
        {
            if (mail == null)
            {
                mail = new Mail(conf, password);
                if (!mail.SetConfigValues())
                {
                    mail = null;
                }
                else
                {
                    string downloadPath = Path.Combine(Path.GetDirectoryName((System.Reflection.Assembly.GetExecutingAssembly().Location)), Mail.downloadDirectoryName);
                    if (!Directory.Exists(downloadPath))
                    {
                        Directory.CreateDirectory(downloadPath);
                    }
                    mail.downloadDirectoryPath = downloadPath;
                }
            }
            return mail;
        }

        public List<MintChipMailData> ConnectToGetPaymentsPosted(List<string> parsedEmailList, bool forceReconnect = false)
        {
            if ((PaidPaymentMessages == null) || (forceReconnect))
            {
                switch (emailType)
                {
                    case "live": LiveConnect(PaymentType.Posted, parsedEmailList); break;
                    case "gmail": GMailImapConnect(PaymentType.Posted, parsedEmailList); break;
                }
            }
            return PaidPaymentMessages;
        }

        public List<MintChipMailData> ConnectToGetPaymentsRequested(bool forceReconnect = false)
        {
            if ((RequestedPaymentMessages == null) || (forceReconnect))
            {
                switch (emailType)
                {
                    case "live": LiveConnect(PaymentType.Requested, null); break;
                    case "gmail": GMailImapConnect(PaymentType.Requested, null); break;
                }
            }
            return RequestedPaymentMessages;
        }

        public string ConnectToSendRequestForPayment(MintChipMailData data)
        {
            MailAddress from = new MailAddress(data.fromEmail);
            MailAddress to = new MailAddress(data.toEmail);
            MailMessage message = new MailMessage(from, to);
            message.Body = data.body;
            message.Body += Environment.NewLine + Environment.NewLine + "New to MintX? Fix that by going here: http://mintxca.com" + Environment.NewLine;
            message.Subject = data.subject;
            FileStream fs = new FileStream(data.attachmentFilePath, FileMode.Open, FileAccess.Read);
            Attachment att = new Attachment(fs, "Request.erq", mimeTypesOfInterest[0]);
            // http://www.afterlogic.com/mailbee-net/docs/MailBee.Mime.Attachment.Filename.html
            att.ContentDisposition.FileName = "Request.erq";
            message.Attachments.Add(att);
            return SendMail(message);
        }

        public string ConnectToSendPayment(MintChipMailData data)
        {
            MailAddress from = new MailAddress(data.fromEmail);
            MailAddress to = new MailAddress(data.toEmail);
            MailMessage message = new MailMessage(from, to);
            message.Body = data.body;
            message.Body += Environment.NewLine + Environment.NewLine + "New to MintX? Fix that by going here: http://mintxca.com" + Environment.NewLine;
            message.Subject = data.subject;
            FileStream fs = new FileStream(data.attachmentFilePath, FileMode.Open, FileAccess.Read);
            Attachment att = new Attachment(fs, "Payment.ecn", mimeTypesOfInterest[1]);
            // http://www.afterlogic.com/mailbee-net/docs/MailBee.Mime.Attachment.Filename.html
            att.ContentDisposition.FileName = "Payment.ecn";
            message.Attachments.Add(att);
            return SendMail(message);
        }

        private Mail(Configuration conf, SecureString password)
        {
            config = conf;
            encryptedPassword = password;
        }

        private bool SetConfigValues()
        {
            try
            {
                emailType = config.AppSettings.Settings["emailType"].Value.ToLowerInvariant();
                emailId = config.AppSettings.Settings["emailId"].Value;
                int.TryParse(config.AppSettings.Settings["LimitOfParsedEmails"].Value, out limit);
                switch (emailType)
                {
                    case "live":
                        smtpHost = config.AppSettings.Settings["LiveSmtpServer"].Value;
                        int.TryParse(config.AppSettings.Settings["LiveSmtpPort"].Value, out smtpPort);
                        popOrImapHost = config.AppSettings.Settings["LivePop3Server"].Value;
                        int.TryParse(config.AppSettings.Settings["LivePop3Port"].Value, out popOrImapPort);
                        break;
                    case "gmail":
                        smtpHost = config.AppSettings.Settings["GmailSmtpServer"].Value;
                        int.TryParse(config.AppSettings.Settings["GmailSmtpPort"].Value, out smtpPort);
                        popOrImapHost = config.AppSettings.Settings["GmailImapServer"].Value;
                        int.TryParse(config.AppSettings.Settings["GmailImapPort"].Value, out popOrImapPort);
                        break;
                }
                extensionsOfInterest = config.AppSettings.Settings["extensionsOfInterestCSV"].Value.Split(new char[] { ',' }, StringSplitOptions.RemoveEmptyEntries).Select(x => x.Trim()).ToArray();
                mimeTypesOfInterest = new string[] { config.AppSettings.Settings["RequestContentType"].Value, config.AppSettings.Settings["ValueMessageType"].Value };
                return ValidateConfigValues();
            }
            catch (Exception)
            {
                return false;
            }
        }

        private bool ValidateConfigValues()
        {
            try
            {
                switch (emailType)
                {
                    case "live": LiveConnect(PaymentType.ValidateOnly, null); break;
                    case "gmail": GMailImapConnect(PaymentType.ValidateOnly, null); break;
                    default: throw new ArgumentOutOfRangeException();
                }
            }
            catch (Exception)
            {
                return false;
            }

            return true;
        }

        // Live works with pop3 which seems to start mail index at 0. Also it only allows you to query the inbox.
        // 3rd party code used: OpenPop
        private void LiveConnect(PaymentType paymentType, List<string> parsedEmailList)
        {
            int end = -1;
            for (int i = 0; i < 3 && end < 0; i++)
            {
                try
                {
                    end = OpenPopClient.GetMessageCount();
                }
                catch (Exception)
                {
                    OpenPopClient = new OpenPop.Pop3.Pop3Client();
                    OpenPopClient.Connect(popOrImapHost, popOrImapPort, true);
                    OpenPopClient.Authenticate(emailId, GetPassword());
                }
            }

            if (end < 0)
            {
                throw new Exception("Unable to connect to the mail server");
            }
            if (paymentType == PaymentType.ValidateOnly) return;
            this.RequestedPaymentMessages = new List<MintChipMailData>();
            this.PaidPaymentMessages = new List<MintChipMailData>();
            SortedSet<string> emails = new SortedSet<string>();
            int start = end - limit > 0 ? end - limit : 1;
            for (int x = start; x <= end; x++)
            {
                OpenPop.Mime.Message msg = OpenPopClient.GetMessage(x);
                if (!string.IsNullOrEmpty(msg.Headers.From.Address))
                    emails.Add(msg.Headers.From.Address);
                List<OpenPop.Mime.MessagePart> attachments = msg.FindAllAttachments();
                if (attachments.Count != 0)
                {
                    foreach (OpenPop.Mime.MessagePart att in attachments)
                    {
                        Dictionary<string, string> attachmentData = new Dictionary<string, string>() {
                                { "attachmentName", att.FileName},
                                { "contentType", att.ContentType.MediaType},
                                { "fileExtension", unknownConst}
                            };

                        try
                        {
                            attachmentData["fileExtension"] = (new FileInfo(att.FileName)).Extension;
                        }
                        catch (Exception) { }

                        if ((extensionsOfInterest.Contains(attachmentData["fileExtension"].ToLowerInvariant())) ||
                            (mimeTypesOfInterest.Contains(attachmentData["contentType"].ToLowerInvariant())))
                        {
                            MintChipMailData data = new MintChipMailData
                            {
                                fromEmail = msg.Headers.From.Address,
                                fromName = msg.Headers.From.DisplayName,
                                onDate = msg.Headers.DateSent,
                                body = msg.FindFirstPlainTextVersion().GetBodyAsText(),
                                messageId = msg.Headers.MessageId,
                                listEntryRepresentation = string.Format("From {0, -40} sent on {1, 10}", msg.Headers.From.Address, msg.Headers.DateSent)
                            };
                            string fileName = Path.Combine(this.downloadDirectoryPath, this.GenerateMessageUniqueName(data));
                            data.attachmentFilePath = fileName;
                            if (File.Exists(fileName))
                            {
                                data.processedAlready = true;
                            }
                            else
                            {
                                data.processedAlready = false;
                                att.Save(new FileInfo(fileName));
                            }
                            if (attachmentData["fileExtension"].ToLowerInvariant() == ".erq")
                            {
                                data.paymentType = MintChipMailData.PaymentType.Requested;
                                this.RequestedPaymentMessages.Add(data);
                            }

                            if (attachmentData["fileExtension"].ToLowerInvariant() == ".ecn")
                            {
                                data.paymentType = MintChipMailData.PaymentType.Paid;
                                this.PaidPaymentMessages.Add(data);
                            }
                        }
                    }
                }
            }
            if (parsedEmailList != null)
            {
                parsedEmailList.AddRange(emails);
            }
        }

        private string GenerateMessageUniqueName(MintChipMailData data)
        {
            System.Text.StringBuilder uniqueStr = new StringBuilder();
            if (!string.IsNullOrEmpty(data.messageId))
                uniqueStr.Append(data.messageId);
            uniqueStr.Append(data.fromEmail).Append(data.onDate).Append(data.body);
            return uniqueStr.ToString().GetHashCode().ToString();
        }

        // gmail works with imap which seems to start mail index at 1.
        // 3rd party library used: AE.Net
        private void GMailImapConnect(PaymentType paymentType, List<string> parsedEmailList)
        {
            AE.Net.Mail.ImapClient ic = new AE.Net.Mail.ImapClient(popOrImapHost, emailId, GetPassword(), AE.Net.Mail.ImapClient.AuthMethods.Login, popOrImapPort, true);
            Mailbox[] mailboxes = ic.ListMailboxes("", "*");
            if (paymentType == PaymentType.ValidateOnly) return;
            this.RequestedPaymentMessages = new List<MintChipMailData>();
            this.PaidPaymentMessages = new List<MintChipMailData>();
            foreach (Mailbox m in mailboxes)
            {
                try
                {
                    ic.SelectMailbox(m.Name);
                    if (m.Name.ToLowerInvariant().IndexOf("inbox") < 0)
                        continue;
                }
                catch (Exception) { continue; }
                int end = ic.GetMessageCount();
                int start = end - limit > 0 ? end - limit : 1;
                AE.Net.Mail.MailMessage[] mm = ic.GetMessages(start, end, false, true);
                SortedSet<string> emails = new SortedSet<string>();
                foreach (AE.Net.Mail.MailMessage msg in mm)
                {
                    if (msg.From != null)
                        emails.Add(msg.From.Address);
                    if (msg.Attachments.Count != 0)
                    {
                        foreach (AE.Net.Mail.Attachment att in msg.Attachments)
                        {
                            Dictionary<string, string> attachmentData = new Dictionary<string, string>() {
                                { "attachmentName", att.Filename},
                                { "contentType", att.ContentType},
                                { "fileExtension", unknownConst}
                            };

                            try
                            {
                                attachmentData["fileExtension"] = (new FileInfo(att.Filename)).Extension;
                            }
                            catch (Exception) { }

                            if ((extensionsOfInterest.Contains(attachmentData["fileExtension"].ToLowerInvariant())) ||
                                (mimeTypesOfInterest.Contains(attachmentData["contentType"].ToLowerInvariant())))
                            {
                                MintChipMailData data = new MintChipMailData
                                {
                                    fromEmail = msg.From.Address,
                                    fromName = msg.From.DisplayName,
                                    onDate = msg.Date,
                                    body = msg.Body,
                                    listEntryRepresentation = string.Format("From {0, -40} sent on {1, 10}", msg.From.DisplayName, msg.Date)
                                };
                                string fileName = Path.Combine(this.downloadDirectoryPath, this.GenerateMessageUniqueName(data));
                                data.attachmentFilePath = fileName;
                                if (File.Exists(fileName))
                                {
                                    data.processedAlready = true;
                                }
                                else
                                {
                                    data.processedAlready = false;
                                    att.Save(fileName);
                                }
                                
                                if (attachmentData["fileExtension"].ToLowerInvariant() == ".erq")
                                {
                                    data.paymentType = MintChipMailData.PaymentType.Requested;
                                    this.RequestedPaymentMessages.Add(data);
                                }

                                if (attachmentData["fileExtension"].ToLowerInvariant() == ".ecn")
                                {
                                    data.paymentType = MintChipMailData.PaymentType.Paid;
                                    this.PaidPaymentMessages.Add(data);
                                }
                            }
                        }
                    }
                }
                if (parsedEmailList != null)
                {
                    parsedEmailList.AddRange(emails);
                }
            }
            ic.Dispose();
        }

        private string GetPassword()
        {
            IntPtr ptr = System.Runtime.InteropServices.Marshal.SecureStringToBSTR(encryptedPassword);
            string s = System.Runtime.InteropServices.Marshal.PtrToStringBSTR(ptr);
            System.Runtime.InteropServices.Marshal.ZeroFreeBSTR(ptr);
            return s;
        }

        private string SendMail(MailMessage message)
        {
            System.Net.Mail.SmtpClient client = new SmtpClient(smtpHost, smtpPort);
            client.EnableSsl = true;
            System.Net.NetworkCredential myCreds = new System.Net.NetworkCredential(emailId, GetPassword(), "");
            client.Credentials = myCreds;
            try
            {
                client.Send(message);
            }
            catch (Exception e)
            {
                return "Sending of message failed: " + e.Message;
            }
            return string.Empty;
        }
    }
}
