﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Security.Cryptography.X509Certificates;
using System.Net;
using System.Net.Security;
using System.IO;
using System.Web.Script.Serialization;
using System.Configuration;
using Mint.MintChip.Core;
using Mint.MintChip.Contract;

namespace MintXCore
{
    //http://developer.mintchipchallenge.com/api/hosted/GetChipInfo.html
    public class RemoteChipHandler : ChipHandler
    {
        //TODO: Confirm that you can access the certificate itself before trying to use it
        // See http://www.codeproject.com/script/Forums/View.aspx?fid=1649&msg=2062983, Look at AddAccessToCertificate
        static X509Certificate2 selectedCert;
        static Uri remoteServer;

        public RemoteChipHandler()
        {
        }

        public override bool Initialize(Configuration conf)
        {
            if ((mintChip != null) && (status != null))
                return true;
            config = conf;
            string certIssuer = config.AppSettings.Settings["RCMCertIssuer"].Value;
            string remote = config.AppSettings.Settings["RemoteRCMServer"].Value;
            LocateCertificate(certIssuer);
            if (selectedCert == null)
            {
                //Console.WriteLine("Could not find certificate to connect to the remote server");
                return false;
            }
            remoteServer = new Uri(remote);
            try
            {
                mintChip = CreateRemoteMintChip();
                status = mintChip.GetStatus();
            }
            catch (MintChipException)
            {
                mintChip = null;
                status = null;
                //Console.WriteLine("Could not load MintChip: " + e.Message);
                return false;
            }
            catch (WebException)
            {
                mintChip = null;
                status = null;
                //Console.WriteLine("Could not load MintChip: " + w.Message);
                return false;
            }
            return true;
        }

        private static void LocateCertificate(string certIssuer)
        {
            X509Store store = new X509Store(StoreName.My, StoreLocation.LocalMachine);
            store.Open(OpenFlags.ReadOnly | OpenFlags.OpenExistingOnly);
            X509Certificate2Collection certCollection = store.Certificates.Find(X509FindType.FindByIssuerDistinguishedName, certIssuer, false);
            X509Certificate2Collection certCollectionFound = new X509Certificate2Collection();
            foreach (X509Certificate2 candidate in certCollection)
            {
                if (candidate.Subject == certIssuer) continue;
                certCollectionFound.Add(candidate);

            }
            if (certCollectionFound.Count == 0) return;
            else if (certCollectionFound.Count == 1)
            {
                selectedCert = certCollectionFound[0];
            }
            else
            {
                //TODO: Support user selecting the certificate
                //Console.WriteLine("Please select the desired certificate:");
                //foreach (X509Certificate2 candidate in certCollectionFound)
                //{
                //    Console.WriteLine(candidate.Subject + " (Thumbprint=" + candidate.Thumbprint + ", Issued on:" + candidate.GetEffectiveDateString() + " Expires on:" + candidate.GetExpirationDateString() + ")");
                //}
                //Console.WriteLine("Selecting the 1st one for you");
                selectedCert = certCollectionFound[0];
            }
        }

        private RemoteChip CreateRemoteMintChip()
        {
            return new RemoteChip(selectedCert, remoteServer.ToString());
        }
    }
}
